/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTFormat.h>
#include <RTLayerConnector.h>
#include <RTMemoryUtil.h>
#include <RTProbe.h>

static const char prompt_prefix[] = "\r\nRTS debug: ";

void RTDebugger::doCmd( void )
{
    enum
    {
	prefix_size = (int)sizeof( prompt_prefix ) - 1,
	count_size  = 40
    };

    int                index;
    int                level  = 0;
    RTController     * task   = (RTController *)0;
    RTActor          * actor;
    char               prompt[ prefix_size + count_size ];
    char             * cursor;

    RTMemoryUtil::memcpy( cursor = prompt, prompt_prefix, prefix_size );
    cursor += prefix_size;

    switch( mode )
    {
    case Halted:  *cursor++ = '-'; break;
    case Running: *cursor++ = '*'; break;
    case Stepping:
	cursor += RTFormat::_unsigned( (unsigned long)step_count,
				       cursor, count_size );
	break;
    default:
	break;
    }
    *cursor++ = '>';
    *cursor++ = '\0';

    input.setPrompt( prompt );

    int command = getCommand();

    RTDiag::out().lock();

    switch( command )
    {
    case CmdNone:
	break;

    case CmdHelp:
	printHelp();
	break;

//  Control based commands CmdExit CmdQuit
    case CmdExit:
	doCmdExit();
	break;

    case CmdQuit:
	doCmdQuit();
	break;

// Message based commands CmdStep and CmdGo
    case CmdStep:
	if( (index = input.getNumber()) <= 0 )
	    index = 1;
	RTDiag::out() << "  step " << index << "\n";
	step( (unsigned)index );
	break;

    case CmdGo:
	if( (index = input.getNumber()) == -1 )
	    index = 10;
	if( index < 1 )
	    RTDiag::out() << "  expecting: go [<num>]\n";
	else
	{
	    RTDiag::out() << "  go " << index << "\n";
	    step( (unsigned)index );
	}
	break;

    case CmdCont:
	RTDiag::out() << "  continue\n";
	proceed();
	break;

    case CmdStats:
	if( (index = input.getNumber()) < 0 )
	    RTDiag::out() << "  Expecting: printstats <num>\n";
	else if( (task = getTask( index )) == (RTController *)0 )
	    RTDiag::out() << "  Task " << index << " does not exist\n";
	else
	    task->printStats();
	break;

    case CmdSaps:
	RTLayerConnector::showSAPs();
	break;

    case CmdSystem:
	doCmdSystem();
	break;

    case CmdInfo:
	if( (actor = getActor()) == (RTActor *)0 )
	    RTDiag::out() << "No such actor\n";
	else
	    printActorInfo( actor );
	break;

// Task based commands CmdTasks CmdAttach CmdDetach
    case CmdTasks:
	if( (task = tasks) != (RTController *)0 )
	{
	    index = 0;
	    do
	    {
		const char * state;

		switch( task->_state )
		{
		case RTController::execUser:
		case RTController::waitDone:
		    state = "stepping ";
		    break;
		case RTController::execSystem:
		case RTController::waitBegin:
		case RTController::waitContinue:
		    state = "stopped  ";
		    break;
		case RTController::detached:
		    state = "detached ";
		    break;
		default:
		    state = "?state?  ";
		    break;
		}

		RTDiag::out()
		    << "  " << index << ": " << state << task->name() << "\n";
	    }
	    while( ++index, (task = task->_next) != tasks );
	}
	break;

    case CmdAttach:
	if( (index = input.getNumber()) < 0 )
	    RTDiag::out() << "  expecting: attach <num>\n";
	else if( (task = getTask( index )) == (RTController *)0 )
	    RTDiag::out() << "  There is no task " << index << "\n";
	else if( task->_state != RTController::detached )
	    RTDiag::out() << "  Task " << index << " already attached\n";
	else
	{
	    attach( task );
	    RTDiag::out() << "  Attached Task " << index << "\n";
	}
	break;

    case CmdDetach:
	if( (index = input.getNumber()) < 0 )
	{
	    RTDiag::out() << "  Detaching all tasks\n";

	    if( (task = tasks) != (RTController *)0 )
	    {
		index = 0;
		do
		{
		    if( task->_state != RTController::detached )
			RTDiag::out() << "  Task " << index << " detached\n";
		    detach( task );
		}
		while( ++index, (task = task->_next) != tasks );
	    }
	}
	else if( (task = getTask( index )) == (RTController *)0 )
	    RTDiag::out() << "  There is no task " << index << "\n";
	else if( task->_state == RTController::detached )
	    RTDiag::out() << "  Task " << index << " already detached\n";
	else
	{
	    detach( task );
	    RTDiag::out() << "  Task " << index << " detached\n";
	}
	break;

    case CmdLog:
	if( (index = getCategory()) >= 0 &&
	    (level = getLevel())    >= 0 )
	{
	    if( index == TraceAll )
		while( --index >= 0 )
		    traceLevel[ index ] = (unsigned char)level;
	    else
		traceLevel[ index ] = (unsigned char)level;
	}
	else
	    RTDiag::out() << "  expecting: log <category> <detail-level>\n";
	break;

    // catch-all command
    case CmdInvalid:
      default:
	RTDiag::out() << "  Invalid command\n";
	break;
    }

    RTDiag::out().unlock();

    input.getEol();
}
